/**********************************************************
*	lcd_demo.c
*
*	Demonstration functions for DEMOHY64 board
*
**********************************************************/

#include <mc9s12hy64.h>				// derivative information
#include "lcd_demo.h"				// demo functions header file
#include "lcddrv.h"					// LCD driver functions
#include "cpu.h"						// CPU fuctions
#include "db_demo.h"					// Dashboard functions



/**********************************************************
*	lcd_on
*
*	turn all LCD segments ON
**********************************************************/
void lcd_on(void){
	Set_Logo(ON, ON, ON, ON, ON);	// display logo and jellyfish icon
	Display_On();						// turn on LCD elements	
}


/**********************************************************
*	LCD_Select_Demo
*	Run Transmission Gear Select and Compass Demo
*
*	control of demo passed thru global variable kbi_state 
*	modified in kbi_isr interrupt routine
**********************************************************/
void LCD_Select_Demo(byte st){
	switch(st){
		case 0:	Display_Off();		// all LCD segments off
					break;
		case 1:	Trans_Demo();		// Transmission segments and labels
					break;
		case 2:	Temp_Demo();		// Temperature segments and labels
					break;
		case 3:	Odo_Demo();			// Odometer segments and labels
					break;
		case 4:	Fuel_Clk_Demo();	// Fuel / Clock segments and labels
					break;
	}
}


/**********************************************************
*	Trans_Demo
*	Run Transmission Gear Select and Compass Demo
*
*	control of demo passed thru global variable kbi_state 
*	modified in kbi_isr interrupt routine
**********************************************************/
void Trans_Demo(void){
	
	byte n;
	
	Display_Off();				// all LCD elements off

	while(menu_sw_st == 2){
		for(n=0; n<8; n++){
			Delay(250);				// wait ms
		   Set_Gear(ON,n);      // roll thru gear selector
			Set_Compass(ON,n);	// roll thru compass points
		}
	}
}

/**********************************************************
*	Temp_Demo
*	Run Temperature Status, Volt Bar, & Temp Bar Demo
*
*	control of demo passed thru global variable kbi_state 
*	modified in kbi_isr interrupt routine
**********************************************************/
void Temp_Demo(void){
	word n;
	byte m, p;
			
	Display_Off();				// all LCD elements off

	while(menu_sw_st == 3){
		for(n=0;n<999; n++){
			m = ((n/4) & 0x03);
			p = ((n/4) & 0x01);
			Set_Temp_Digits(n);

			switch((n/6) & 0x01){			// select Temp scale indicator to turn on
				case 0:	DEG_F = ON;
							DEG_C = OFF;
							TEMP_DEC = ON;	
							break;
				case 1:	DEG_F = OFF;
							DEG_C = ON;	
							TEMP_DEC = OFF;	
							break;
			}

			switch(m){							// select Temp indicator label to turn on
				case 0:	INSIDE = ON;
							TEMP = OFF;
							break;
				case 1:	OUTSIDE = ON;
							INSIDE = OFF;
							break;
				case 2:	ENGINE = ON;
							OUTSIDE = OFF;
							break;
				case 3:	TEMP = ON;
							ENGINE = OFF;
							break;
			}

			Set_Volt_Temp_Bars();				// display Volt & Temp bars
		
				Delay(100);
			}
	}
}


/**********************************************************
*	Odo_Demo	
*	Run Odometer / Trip LCD Segments Demo
*
*	control of demo passed thru global variable kbi_state 
*	modified in kbi_isr interrupt routine
**********************************************************/
void Odo_Demo(void){			// display fuel & clock segments
	word temp_ctr = 0;
	
	Display_Off();				// all LCD elements off

	while(menu_sw_st == 4){
		for(temp_ctr=0; temp_ctr<16; temp_ctr++){
			Delay(250);								// Delay to enhance display readability
			
			Set_ODO_Labels_All_Off();			// ODO indicators off

			switch((temp_ctr/2) & 0x03){
				case 0: SEG_P40 = ON; break;	// ODO indicator on
				case 1: SEG_P41 = ON; break;	// TRIPA indicator on
				case 2: SEG_P42 = ON; break;	// TRIPB indicator on
				case 3: SEG_R4  = ON; break;	// ODO decimal on	
			}
			
			switch ((temp_ctr/4) & 0x01){
				case 0: SEG_P34 = ON; break;	// MILES indicator on
				case 1: SEG_P35 = ON; break;	// KM indicator on
			}

			if((temp_ctr>0) && (temp_ctr<10)) Set_Digit_13(temp_ctr);
				else Set_Digit_13(0);
			if((temp_ctr>1) && (temp_ctr<11)) Set_Digit_12(temp_ctr-1);
				else Set_Digit_12(0);
			if((temp_ctr>2) && (temp_ctr<12)) Set_Digit_11(temp_ctr-2);
				else Set_Digit_11(0);
			if((temp_ctr>3) &&(temp_ctr<13)) Set_Digit_10(temp_ctr-3);
				else Set_Digit_10(0);
			if((temp_ctr>4) && (temp_ctr<14)) Set_Digit_9(temp_ctr-4);
				else Set_Digit_9(0);
			if((temp_ctr>5) && (temp_ctr<15)) Set_Digit_8(temp_ctr-5);
				else Set_Digit_8(0);
		}
	}
}


/**********************************************************
*	Fuel_Clk_Demo
*	Run Fuel/Clock/Status LCD Segments Demo
*
*	control of demo passed thru global variable kbi_state 
*	modified in kbi_isr interrupt routine
**********************************************************/
void Fuel_Clk_Demo(void){			// display fuel / clock segments
//	word temp_ctr, local_state = 0;
	word n;
		
	Display_Off();				// all LCD elements off

	if(menu_sw_st == 5){
	   Set_Cruise(ON);
	   Set_Traction(ON);
		} 
		
	while(menu_sw_st == 5){
		for(n=0; n<0x270F; n++){
			Set_Fuel_Clock_Digits(n);
			Set_Fuel_Clock_Labels((n/6) & 0x0F);
			Set_Status_Labels((n/12) & 0x03); 
			Delay(100);
		}
	}
}


/**********************************************************
*	Set_Volt_Temp_Bars
*	
*  display Volt and Temp bars on LCD
**********************************************************/
void Set_Volt_Temp_Bars(void){
	
	byte temp, r;
	
	temp = read_atd_ch3();			// get POT reading
	r = ((temp/0x2A) & 0x07);		// parse reading into steps
	if(r > 6) r = 6;
	
	Set_Volt_Bar(ON,r);			   // display Volt bars
	Set_Temp_Bar(ON,6-r);			// display Temp bars
	
}






