/**********************************************************
*
*	FILE				: motor_drv.c
*
*	AUTHOR			: Axiom Manufacturing
*						  Jan 2009    					     
*						  
*	PURPOSE			: 9S12HY64 Motor Controller functions based on 
*						  MC10B8CV1 Motor Controller Block Description
**********************************************************/

#include <hidef.h>					// common defines and macros
#include <mc9s12hy64.h>				// derivative information
#include "motor_drv.h"				// Motor Driver header file
#include "cpu.h"						// CPU header file
#include "db_demo.h"					// Demo board header file

word num_steps;

/**********************************************************
*	MC_init
*
*	initialize the Motor Controller block.  Config for Dual 
*	Full-Bridge operation and set pre-scalar.  
**********************************************************/
void MC_init(void){
	
	MCCTL0_MCPRE = DIV8;				// load pre-scalar value
	MCCTL0_FAST = 0;					// 11-bit resolution
	MCCTL1_RECIRC = 0;				// recirc in high-side transistors
//	MCCTL1_MCTOIE = 1;				// enable timer overflow interrupt
	
	MCCC0_MCOM = DUALFULLBRIDGE;	// Motor1 set to dual full h-bridge configuration
	MCCC1_MCOM = DUALFULLBRIDGE;
	MCCC2_MCOM = DUALFULLBRIDGE;	// Motor2 set to dual full h-bridge configuration
	MCCC3_MCOM = DUALFULLBRIDGE;
	MCCC4_MCOM = DUALFULLBRIDGE;	// Motor3 set to dual full h-bridge configuration
	MCCC5_MCOM = DUALFULLBRIDGE;
	MCCC6_MCOM = DUALFULLBRIDGE;	// Motor4 set to dual full h-bridge configuration
	MCCC7_MCOM = DUALFULLBRIDGE;
	
	MCDC0 = 0x554;						// 2/3 duty cycle, S = 0
	MCDC1 = 0x554;						// 2/3 duty cycle, S = 0
	MCDC2 = 0x554;						// 2/3 duty cycle, S = 0
	MCDC3 = 0x554;						// 2/3 duty cycle, S = 0
	MCDC4 = 0x554;						// 2/3 duty cycle, S = 0
	MCDC5 = 0x554;						// 2/3 duty cycle, S = 0
	MCDC6 = 0x554;						// 2/3 duty cycle, S = 0
	MCDC7 = 0x554;						// 2/3 duty cycle, S = 0

	MCCTL0_MCTOIF = 1;				// clear overflow flag

//	MC_zero_pointer();				// motor pointer to zero position

	MPH_LED = OFF;						// ensure motor LED is off 
	RPM_LED = OFF;						// ensure motor LED is off 
	FUEL_LED = OFF;					// ensure motor LED is off 
	TEMP_LED = OFF;					// ensure motor LED is off 
}


/**********************************************************
*	MC_zero_pointer
*
*	move motor pointer CCW to zero position.  called at 
* 	application start.  configured for each motor individually
**********************************************************/
void MC_zero_pointer(void){

	MC_step_dir_all(CW);				// set all motors to move CCW
	MC_step_all(10);
	MC_step_dir_all(CCW);				// set all motors to move CW
	MC_step_all(300);					// move all pointers to max CCW position
	MC_step_dir_all(CW);

	MC_step_mph_motor(20);			// move pointer to 0 position
	MC_step_rpm_motor(15);			// move pointer to 0 position
	MC_step_fuel_motor(16);			// move pointer to 0 position
	MC_step_temp_motor(23);			// move pointer to 0 position

}


/**********************************************************
*	MC_step_dir
*
*	Setup motor controller step direction 
*
*	INPUT:	dir 0 - CW: 1 - CCW
**********************************************************/
void MC_step_dir_all(byte dir){

	if(dir){						// set all motors to CCW direction
		MPH_STEP_CCW;
		RPM_STEP_CCW;
		FUEL_STEP_CCW;
		TEMP_STEP_CCW;
	} else{						// set all motors to CW direction
		MPH_STEP_CW;
		RPM_STEP_CW;
		FUEL_STEP_CW;
		TEMP_STEP_CW;
	}
}


/**********************************************************
*	MC_step_dir
*
*	Setup motor controller step direction 
*
*	INPUT:	motor	1 - MPH, 2 - RMP, 3 - FUEL, 4 - TEMP
*				dir	0 - CW: 1 - CCW
**********************************************************/
void MC_step_dir(byte motor, byte dir){

	switch(motor){
		case 1:							// MPH motor (M1) direction
			if(dir){						// CCW direction
				MPH_STEP_CCW;
			} else{						// CW direction
				MPH_STEP_CW;
			}
			break;
		case 2:							// RPM motor (M2) direction
			if(dir){						// CCW direction
				RPM_STEP_CCW;
			} else{						// CW direction
				RPM_STEP_CW;
			}
			break;
		case 3:							// FUEL motor (M3) direction
			if(dir){						// CCW direction
				FUEL_STEP_CCW;
			} else{						// CW direction
				FUEL_STEP_CW;
			}
			break;
		case 4:							// TEMP motor (M4) direction
			if(dir){						// CCW direction
				TEMP_STEP_CCW;
			} else{						// CW direction
				TEMP_STEP_CW;
			}
			break;
		default:	break;					// exit on other input
	}
}


/**********************************************************
*	MC_step_all
*
*	step motor indicated number of steps.  input limited to 
*	320 steps to prevent motor damage 
*
*	Input:	#steps to move pointer	
**********************************************************/
void MC_step_all(word step){

	word n;
	if(step > 0x140){
		num_steps = MAX_STEP_CNT;	// limit # of steps to max count
	} else num_steps = step;		// get step input
	
	MCPER = MC_PERIOD;				// start motor controller counter
	for(n=0; n<num_steps; n++){
		while(!MCCTL0_MCTOIF);		// wait for MC timer to overflow
		MCDC0 ^= 0x8000;				// toggle S-bit for motor 0, Ch 0
		MCDC1 ^= 0x8000;				// toggle S-bit for motor 0, Ch 1
		MCDC2 ^= 0x8000;				// toggle S-bit for motor 0, Ch 2
		MCDC3 ^= 0x8000;				// toggle S-bit for motor 0, Ch 3
		MCDC4 ^= 0x8000;				// toggle S-bit for motor 0, Ch 4
		MCDC5 ^= 0x8000;				// toggle S-bit for motor 0, Ch 5
		MCDC6 ^= 0x8000;				// toggle S-bit for motor 0, Ch 6
		MCDC7 ^= 0x8000;				// toggle S-bit for motor 0, Ch 7
		MCCTL0_MCTOIF =1;				// clear flag
	}
	MCPER = MC_PER_OFF;				// disable MC
}


/**********************************************************
*	MC_step_mph_motor
*
*	Step MPH motor (M1) indicated number of steps.  To prevent 
*	motor damage, input step count is limited to320 steps 
*
*	Input:	#steps to move pointer	
**********************************************************/
void MC_step_mph_motor(word step){

	word n;
	if(step > 0x140){
		num_steps = MAX_STEP_CNT;	// limit # of steps to max count
	} else num_steps = step;		// get step input
	
	MCPER = MC_PERIOD;				// start motor controller counter
	for(n=0; n<num_steps; n++){
		while(!MCCTL0_MCTOIF);		// wait for MC timer to overflow
		MCDC0 ^= 0x8000;				// toggle S-bit for motor 0, Ch 0
		MCDC1 ^= 0x8000;				// toggle S-bit for motor 0, Ch 1
		MCCTL0_MCTOIF =1;				// clear flag
	}
	MCPER = MC_PER_OFF;				// disable MC
}


/**********************************************************
*	MC_step_fuel_motor
*
*	Step RPM motor (M3) indicated number of steps.  To prevent 
*	motor damage, input step count is limited to320 steps 
*
*	Input:	#steps to move pointer	
**********************************************************/
void MC_step_fuel_motor(word step){

	word n;
	if(step > 0x140){
		num_steps = MAX_STEP_CNT;	// limit # of steps to max count
	} else num_steps = step;		// get step input
	
	MCPER = MC_PERIOD;				// start motor controller counter
	for(n=0; n<num_steps; n++){
		while(!MCCTL0_MCTOIF);		// wait for MC timer to overflow
		MCDC2 ^= 0x8000;				// toggle S-bit for motor 0, Ch 2
		MCDC3 ^= 0x8000;				// toggle S-bit for motor 0, Ch 3
		MCCTL0_MCTOIF =1;				// clear flag
	}
	MCPER = MC_PER_OFF;				// disable MC
}


/**********************************************************
*	MC_step_rpm_motor
*
*	Step RPM motor (M2) indicated number of steps.  To prevent 
*	motor damage, input step count is limited to320 steps 
*
*	Input:	#steps to move pointer	
**********************************************************/
void MC_step_rpm_motor(word step){

	word n;
	if(step > 0x140){
		num_steps = MAX_STEP_CNT;	// limit # of steps to max count
	} else num_steps = step;		// get step input
	
	MCPER = MC_PERIOD;				// start motor controller counter
	for(n=0; n<num_steps; n++){
		while(!MCCTL0_MCTOIF);		// wait for MC timer to overflow
		MCDC4 ^= 0x8000;				// toggle S-bit for motor 0, Ch 4
		MCDC5 ^= 0x8000;				// toggle S-bit for motor 0, Ch 5
		MCCTL0_MCTOIF =1;				// clear flag
	}
	MCPER = MC_PER_OFF;				// disable MC
}


/**********************************************************
*	MC_step_temp_motor
*
*	Step RPM motor (M4) indicated number of steps.  To prevent 
*	motor damage, input step count is limited to320 steps 
*
*	Input:	#steps to move pointer	
**********************************************************/
void MC_step_temp_motor(word step){

	word n;
	if(step > 0x140){
		num_steps = MAX_STEP_CNT;	// limit # of steps to max count
	} else num_steps = step;		// get step input
	
	MCPER = MC_PERIOD;				// start motor controller counter
	for(n=0; n<num_steps; n++){
		while(!MCCTL0_MCTOIF);		// wait for MC timer to overflow
		MCDC6 ^= 0x8000;				// toggle S-bit for motor 0, Ch 6
		MCDC7 ^= 0x8000;				// toggle S-bit for motor 0, Ch 7
		MCCTL0_MCTOIF =1;				// clear flag
	}
	MCPER = MC_PER_OFF;				// disable MC
}


/**********************************************************
*	MC_motor_leds
*
*	rotate motor 1 in CW & CCW alternately 
*
**********************************************************/
void MC_motor_leds(byte st){

	MPH_LED = st;						// turn MPH motor LED on 
	RPM_LED = st;						// turn FUEL motor LED on 
	FUEL_LED = st;						// turn RPM motor LED on 
	TEMP_LED = st;						// turn TEMP motor LED on 
}


/**********************************************************
*	MC_motor_demo
*
*	rotate motor 1 in CW & CCW alternately 
*
**********************************************************/
void MC_motor_demo(void){

	MPH_LED = ON;						// turn MPH motor LED on 
	RPM_LED = ON;						// turn FUEL motor LED on 
	FUEL_LED = ON;						// turn RPM motor LED on 
	TEMP_LED = ON;						// turn TEMP motor LED on 

	for(;;){
		MC_step_dir_all(motor_dir);		
		MC_step_all(245);
		motor_dir ^= 1;				// 0 = CW, 1 = CCW
		Delay(100);						// add delay
	}
}


/*
#pragma CODE_SEG NON_BANKED
*********************************************************
*	MC_Timer_Isr
*
*	Motor Controller (MC) interrupt service routine
*********************************************************
void interrupt 52 MC_Timer_Isr(void){

	MCDC0 ^= 0x8000;					// toggle S bit
	MCDC1 ^= 0x8000;					// toggle S bit

	if(step_count == num_steps){
		step_count = 0;
		MCPER = MC_PER_OFF;			// disable MC counter
		step_done = 1;					// set step complete flag
	} else step_count++;				// else incr step_count

	MCCTL0_MCTOIF = 1;				// clear overflow flag
}
#pragma CODE_SEG DEFAULT
*/